package com.vcarecity.test.service.elastic;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.inject.Inject;
import com.vcarecity.elastic.util.AgencyPathUtil;
import com.vcarecity.test.entity.UnitAgencyDetailEntity;
import com.vcarecity.test.entity.UnitAgencyEntity;
import com.vcarecity.test.service.AgencyService;
import com.vcarecity.test.service.ElasticSaveService;
import com.vcarecity.test.util.SnowFlake;
import lombok.extern.slf4j.Slf4j;
import net.sf.cglib.beans.BeanCopier;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.xcontent.XContentType;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Kerry on 19/12/09
 */

@Slf4j
public class ElasticSaveServiceImpl implements ElasticSaveService {

    private final RestHighLevelClient restHighLevelClient;
    private final AgencyService agencyService;
    private final ObjectMapper objectMapper;

    @Inject
    public ElasticSaveServiceImpl(RestHighLevelClient restHighLevelClient,
                                  AgencyService agencyService,
                                  ObjectMapper objectMapper) {
        this.restHighLevelClient = restHighLevelClient;
        this.agencyService = agencyService;
        this.objectMapper = objectMapper;
    }

    @Override
    public <T> void saveUnitAgencyData(List<UnitAgencyEntity> result) {

        final BeanCopier beanCopier = BeanCopier.create(UnitAgencyEntity.class, UnitAgencyDetailEntity.class, false);

        List<Map> saveResult = new ArrayList<>(result.size());


            for (UnitAgencyEntity uae : result) {
            final List<Integer> agencyIdList = AgencyPathUtil.splitAgencyId(uae.getAgencyPath());
            List<String> agencyPathDetail = agencyService.getAgencyPathDetail(agencyIdList);

            UnitAgencyDetailEntity unitAgencyDetailEntity = new UnitAgencyDetailEntity();

            beanCopier.copy(uae, unitAgencyDetailEntity, null);

            unitAgencyDetailEntity.setAgencyDetail(String.join("", agencyPathDetail));

            @SuppressWarnings("rawtypes")
            Map map = objectMapper.convertValue(unitAgencyDetailEntity, Map.class);
            final Map<String, Integer> queryMap = AgencyPathUtil.agencyPathQueryMap(agencyIdList);
            //noinspection unchecked
            map.putAll(queryMap);

            saveResult.add(map);
        }


        String index = "unit-agency";
        logger.info("start insert into: {}", index);

        try {
            BulkRequest bulkRequest = new BulkRequest();
            boolean hasData = false;
            for (Map val : saveResult) {
                if (!hasData) {
                    hasData = true;
                }
                IndexRequest indexRequest = createIndexRequest(index, val);
                /// restHighLevelClient.index(indexRequest, RequestOptions.DEFAULT);
                bulkRequest.add(indexRequest);
            }
            if (hasData) {
                long time = System.currentTimeMillis();
                BulkResponse bulkResponse = restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT);
                logger.info("bulk insert status {}. usage: {} /ms", bulkResponse.status(), (System.currentTimeMillis() - time));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @SuppressWarnings("rawtypes")
    private IndexRequest createIndexRequest(String index, Map value) {
        IndexRequest indexRequest = new IndexRequest(index);
        indexRequest.id(SnowFlake.nextId() + "");
        //noinspection unchecked
        indexRequest.source(value, XContentType.JSON);
        return indexRequest;
    }

}
