package com.vcarecity.test.service.oracle;

import com.google.inject.Inject;
import com.vcarecity.test.entity.UnitAgencyEntity;
import com.vcarecity.test.service.ObtainDataService;
import com.vcarecity.test.typehandler.TypeHandler;
import lombok.extern.slf4j.Slf4j;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * @author Kerry on 19/12/09
 */

@Slf4j
public class OracleObtainDataServiceImpl implements ObtainDataService {

    public static final String OB_SQL_1 = "select P.*\n" +
            "from (\n" +
            "         SELECT TU.UNIT_ID,\n" +
            "                TU.DTU_NO,\n" +
            "                TU.PUSER_CODE,\n" +
            "                TAG.AGENCY_ID,\n" +
            "                TAG.AGENCY_PATH,\n" +
            "                TAG.AGENCY_NAME,\n" +
            "                TB.BUILDING_ID,\n" +
            "                TB.NAME_OF_BUILDING,\n" +
            "                TUT.UNIT_TYPE_ID,\n" +
            "                TUT.UNIT_TYPE,\n" +
            "                TE.EVENT,\n" +
            "                ROW_NUMBER() OVER (ORDER BY TUT.PROPERTY , TU.UPDATESTAMP DESC, TU.UNIT_ID) RN\n" +
            "         FROM T_AGENCY TAG\n" +
            "                  INNER JOIN T_UNIT TU ON TU.AGENCY_ID = TAG.AGENCY_ID AND TU.ISDELETED = 0\n" +
            "                  INNER JOIN T_BUILDING TB ON TB.BUILDING_ID = TU.BUILDING_ID\n" +
            "                  INNER JOIN T_UNIT_TYPE TUT ON TUT.UNIT_TYPE_ID = TU.UNIT_TYPE_ID\n" +
            "                  LEFT JOIN T_EVENT TE ON TE.EVENT_ID = TU.UNIT_STATE_ID\n" +
            "     ) P\n" +
            "WHERE P.RN > ? " +
            "  AND P.RN <= ?";


    public static final String OB_SQL_2 = "select P.*\n" +
            "from (\n" +
            "         SELECT TU.UNIT_ID,\n" +
            "                TU.DTU_NO,\n" +
            "                TU.PUSER_CODE,\n" +
            "                TAG.AGENCY_ID,\n" +
            "                TAG.AGENCY_PATH,\n" +
            "                TAG.AGENCY_NAME,\n" +
            "                TB.BUILDING_ID,\n" +
            "                TB.NAME_OF_BUILDING,\n" +
            "                TUT.UNIT_TYPE_ID,\n" +
            "                TUT.UNIT_TYPE,\n" +
            "                TE.EVENT,\n" +
            "                ROWNUM RN\n" +
            "         FROM T_AGENCY TAG\n" +
            "                  INNER JOIN T_UNIT TU ON TU.AGENCY_ID = TAG.AGENCY_ID AND TU.ISDELETED = 0\n" +
            "                  INNER JOIN T_BUILDING TB ON TB.BUILDING_ID = TU.BUILDING_ID\n" +
            "                  INNER JOIN T_UNIT_TYPE TUT ON TUT.UNIT_TYPE_ID = TU.UNIT_TYPE_ID\n" +
            "                  LEFT JOIN T_EVENT TE ON TE.EVENT_ID = TU.UNIT_STATE_ID\n" +
            "         where ROWNUM <= ?\n" +
            "     ) P\n" +
            "WHERE P.RN > ?";


    public static final String OB_SQL = "select P.*\n" +
            "from (\n" +
            "         SELECT TU.UNIT_ID,\n" +
            "                TU.DTU_NO,\n" +
            "                TU.PUSER_CODE,\n" +
            "                TAG.AGENCY_ID,\n" +
            "                TAG.AGENCY_PATH,\n" +
            "                TAG.AGENCY_NAME,\n" +
            "                TB.BUILDING_ID,\n" +
            "                TB.NAME_OF_BUILDING,\n" +
            "                TUT.UNIT_TYPE_ID,\n" +
            "                TUT.UNIT_TYPE,\n" +
            "                ROWNUM RN\n" +
            "         FROM T_UNIT TU\n" +
            "                  INNER JOIN T_AGENCY TAG ON TU.AGENCY_ID = TAG.AGENCY_ID AND TU.ISDELETED = 0\n" +
            "                  INNER JOIN T_BUILDING TB ON TB.BUILDING_ID = TU.BUILDING_ID\n" +
            "                  INNER JOIN T_UNIT_TYPE TUT ON TUT.UNIT_TYPE_ID = TU.UNIT_TYPE_ID\n" +
            "         where ROWNUM <= ?\n" +
            "     ) P\n" +
            "WHERE P.RN > ?";

    private final DataSource dataSource;
    private final TypeHandler typeHandler;

    @Inject
    public OracleObtainDataServiceImpl(DataSource dataSource,
                                       TypeHandler typeHandler) {
        this.dataSource = dataSource;
        this.typeHandler = typeHandler;
        logger.info("OracleObtainDataServiceImpl.OracleObtainDataServiceImpl. dataSource = {}", dataSource);
    }


    @Override
    public List<UnitAgencyEntity> queryUnitAgencyDataByPage(int page, int size) {

        logger.debug("SQL = {}. page = {}, size = {}", OB_SQL, page, size);

        int begin = page * size;
        int end = (1 + page) * size;

        try (Connection connection = dataSource.getConnection();
             final PreparedStatement ps = connection.prepareStatement(OB_SQL)) {
            ps.setInt(1, end);
            ps.setInt(2, begin);

            try (ResultSet resultSet = ps.executeQuery()) {
                return typeHandler.resultHandler(resultSet, UnitAgencyEntity.class);
            } catch (IllegalAccessException | InstantiationException e) {
                e.printStackTrace();
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }
}
