package com.vcarecity.test.task;

import com.google.inject.Inject;
import com.vcarecity.test.HighClientApp;
import com.vcarecity.test.entity.UnitAgencyEntity;
import com.vcarecity.test.service.ElasticSaveService;
import com.vcarecity.test.service.ObtainDataService;
import com.zaxxer.hikari.HikariDataSource;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.client.RestHighLevelClient;

import javax.sql.DataSource;
import java.util.List;

/**
 * @author Kerry on 19/12/09
 */

@Slf4j
public class ImportTask {

    private static final int PAGE_COUNT = 2000;

    private final ObtainDataService obtainDataService;
    private final ElasticSaveService elasticSaveService;

    @Inject
    public ImportTask(ObtainDataService obtainDataService,
                      ElasticSaveService elasticSaveService) {
        this.obtainDataService = obtainDataService;
        this.elasticSaveService = elasticSaveService;
    }


    public void startTask() {
        logger.info("ImportTask.startTask. {}", obtainDataService.getClass());

        long allTime = System.currentTimeMillis();

        int count = 0;

        int start = 0;
        List<UnitAgencyEntity> result;
        do {
            long startTime = System.currentTimeMillis();
            result = obtainDataService.queryUnitAgencyDataByPage(start++, PAGE_COUNT);
            count += result.size();
            logger.info("query usageTime: {} ms. count = {}", (System.currentTimeMillis() - startTime), count);

            startTime = System.currentTimeMillis();
            elasticSaveService.saveUnitAgencyData(result);

            logger.info("write to Elasticsearch usageTime: {} ms.", (System.currentTimeMillis() - startTime));

        } while (result.size() == PAGE_COUNT);

        logger.info("ImportTask.stop. All usageTime = {} ms.", (System.currentTimeMillis() - allTime));

    }

    public void stopApp() {
        try {
            final DataSource instance = HighClientApp.injector.getInstance(DataSource.class);
            ((HikariDataSource) instance).close();
        } catch (Exception e) {

        }
        try {
            final RestHighLevelClient instance = HighClientApp.injector.getInstance(RestHighLevelClient.class);
            instance.close();
        } catch (Exception e) {

        }

    }
}
