package com.vcarecity.publish.elastic.service.impl;

import com.vcarecity.publish.constants.ElasticConstant;
import com.vcarecity.publish.elastic.service.ElasticLoadDataService;
import com.vcarecity.publish.elastic.service.ElasticUpdateService;
import com.vcarecity.publish.entity.UnitAgencyMergeEntity;
import com.vcarecity.publish.mapper.UnitMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;

/**
 * @author kerryzhang on 2019/12/17
 */

@Slf4j
@Service
public class ElasticLoadDataServiceImpl implements ElasticLoadDataService {


    private final UnitMapper unitMapper;
    private final ElasticUpdateService elasticUpdateService;
    private final int pageCount;

    public ElasticLoadDataServiceImpl(UnitMapper unitMapper,
                                      ElasticUpdateService elasticUpdateService) {
        this.unitMapper = unitMapper;
        this.elasticUpdateService = elasticUpdateService;
        this.pageCount = ElasticConstant.QUERY_PAGE_COUNT;
    }


    @Override
    public void loadData() throws IOException {

        long allTime = System.currentTimeMillis();

        // calc handler count
        int count = 0;
        // page
        int start = 0;

        List<UnitAgencyMergeEntity> result;
        do {

            long startTime = System.currentTimeMillis();

            result = unitMapper.findAllWithPage(start * pageCount, pageCount);
            start++;
            count += result.size();

            logger.debug("query database usageTime: {} ms. count = {}", (System.currentTimeMillis() - startTime), count);

            startTime = System.currentTimeMillis();

            elasticUpdateService.addElasticData(result);

            logger.debug("write to Elasticsearch usageTime: {} ms.", (System.currentTimeMillis() - startTime));

        } while (result.size() == pageCount);

        logger.info("loadData ok. All usageTime = {} ms.", (System.currentTimeMillis() - allTime));

    }

}
