package com.vcarecity.publish.task;

import com.vcarecity.publish.constants.ElasticConstant;
import com.vcarecity.publish.elastic.service.ElasticUpdateService;
import com.vcarecity.publish.entity.UnitAgencyMergeEntity;
import com.vcarecity.publish.mapper.AgencyMapper;
import com.vcarecity.publish.mapper.UnitMapper;
import com.vcarecity.publish.service.CheckTimeService;
import com.vcarecity.publish.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;

/**
 * @author Kerry on 19/12/11
 */

@Slf4j
@Component
public class CheckUpdateTask {

    private final CheckTimeService checkTimeService;
    private final ElasticUpdateService elasticUpdateService;
    private final AgencyMapper agencyMapper;
    private final UnitMapper unitMapper;


    public CheckUpdateTask(CheckTimeService checkTimeService,
                           ElasticUpdateService elasticUpdateService,
                           AgencyMapper agencyMapper,
                           UnitMapper unitMapper) {
        this.checkTimeService = checkTimeService;
        this.elasticUpdateService = elasticUpdateService;
        this.agencyMapper = agencyMapper;
        this.unitMapper = unitMapper;
    }


    @Scheduled(initialDelay = 1000L, fixedDelay = ElasticConstant.CHECK_TIME_INTERVAL)
    public void checkUpdateTask() {

        // 当前更新时间
        long curCheckTime = System.currentTimeMillis();
        String toDateTime = LocalDateTime.ofInstant(new Date(curCheckTime).toInstant(), ZoneId.systemDefault()).format(DateUtil.DATE_TIME_FORMATTER);


        List<UnitAgencyMergeEntity> updateUaList;
        String fromDateTime;

        // 获取上一次的时间
        fromDateTime = checkTimeService.getLastDateTime("T_UNIT");

        logger.info("start checkUpdateTask on T_UNIT from:{}. to:{}", fromDateTime, toDateTime);

        try {
            updateUaList = unitMapper.findUpdateUaList(fromDateTime, toDateTime);

            logger.debug("T_UNIT update list size = {}", updateUaList.size());

            if (!updateUaList.isEmpty()) {
                elasticUpdateService.handlerDataChange("unitId", updateUaList);
            }
            checkTimeService.setLastDateTime("T_UNIT", curCheckTime);
            updateUaList.clear();
        } catch (Exception e) {
            e.printStackTrace();
        }

        fromDateTime = checkTimeService.getLastDateTime("T_AGENCY");

        logger.info("start checkUpdateTask on T_AGENCY from:{}. to:{}", fromDateTime, toDateTime);

        try {
            updateUaList = agencyMapper.findUpdateUaList(fromDateTime, toDateTime);

            logger.debug("T_AGENCY update list size = {}", updateUaList.size());

            if (!updateUaList.isEmpty()) {
                elasticUpdateService.handlerDataChange("agencyId", updateUaList);
            }
            checkTimeService.setLastDateTime("T_AGENCY", curCheckTime);
            updateUaList.clear();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
